var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { isNull, isUndefined } from 'lodash';
import React, { useCallback, useState, useMemo, useRef, useImperativeHandle } from 'react';
import { useEventCallback, useId } from '@react-devui/hooks';
import { CloseOutlined, LoadingOutlined } from '@react-devui/icons';
import { findNested, getClassName, getVerticalSidePosition } from '@react-devui/utils';
import { useGeneralContext, useDValue } from '../../hooks';
import { cloneHTMLElement, registerComponentMate, TTANSITION_DURING_POPUP, WINDOW_SPACE } from '../../utils';
import { DComboboxKeyboard } from '../_keyboard';
import { DSelectbox } from '../_selectbox';
import { DTransition } from '../_transition';
import { DDropdown } from '../dropdown';
import { useFormControl } from '../form';
import { useComponentConfig, usePrefixConfig, useTranslation } from '../root';
import { DTag } from '../tag';
import { DSearchPanel as DTreeSearchPanel } from '../tree/SearchPanel';
import { MultipleTreeNode } from '../tree/multiple-node';
import { SingleTreeNode } from '../tree/single-node';
import { getText, TREE_NODE_KEY } from '../tree/utils';
import { DList } from './List';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DCascader' });
function Cascader(props, ref) {
    var _a;
    const _b = useComponentConfig(COMPONENT_NAME, props), { dRef, dFormControl, dList, dModel, dVisible, dPlaceholder, dSize, dLoading = false, dSearchable = false, dSearchValue, dClearable = false, dDisabled = false, dMultiple = false, dOnlyLeafSelectable = true, dVirtual = false, dCustomItem, dCustomSelected, dCustomSearch, dPopupClassName, dInputRender, onModelChange, onVisibleChange, onSearchValueChange, onClear, onFirstFocus, afterVisibleChange } = _b, restProps = __rest(_b, ["dRef", "dFormControl", "dList", "dModel", "dVisible", "dPlaceholder", "dSize", "dLoading", "dSearchable", "dSearchValue", "dClearable", "dDisabled", "dMultiple", "dOnlyLeafSelectable", "dVirtual", "dCustomItem", "dCustomSelected", "dCustomSearch", "dPopupClassName", "dInputRender", "onModelChange", "onVisibleChange", "onSearchValueChange", "onClear", "onFirstFocus", "afterVisibleChange"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { gSize, gDisabled } = useGeneralContext();
    //#endregion
    //#region Ref
    const boxRef = useRef(null);
    const popupRef = useRef(null);
    const comboboxKeyDownRef = useRef(null);
    //#endregion
    const dataRef = useRef({
        focusList: new Set(),
    });
    const [t] = useTranslation();
    const uniqueId = useId();
    const listId = `${dPrefix}cascader-list-${uniqueId}`;
    const getItemId = (val) => `${dPrefix}cascader-item-${val}-${uniqueId}`;
    const renderNodes = useMemo(() => dList.map((item) => dMultiple
        ? new MultipleTreeNode(item, (origin) => origin.value, {
            disabled: item.disabled,
        })
        : new SingleTreeNode(item, (origin) => origin.value, {
            disabled: item.disabled,
        })), [dMultiple, dList]);
    const nodesMap = useMemo(() => {
        const nodes = new Map();
        const reduceArr = (arr) => {
            for (const item of arr) {
                nodes.set(item.id, item);
                if (item.children) {
                    reduceArr(item.children);
                }
            }
        };
        reduceArr(renderNodes);
        return nodes;
    }, [renderNodes]);
    const [searchValue, changeSearchValue] = useDValue('', dSearchValue, onSearchValueChange);
    const [visible, changeVisible] = useDValue(false, dVisible, onVisibleChange);
    const formControlInject = useFormControl(dFormControl);
    const [_select, changeSelect] = useDValue(dMultiple ? [] : null, dModel, (value) => {
        var _a;
        if (onModelChange) {
            if (dMultiple) {
                onModelChange(value, value.map((v) => { var _a; return (_a = nodesMap.get(v)) === null || _a === void 0 ? void 0 : _a.origin; }));
            }
            else {
                onModelChange(value, isNull(value) ? null : (_a = nodesMap.get(value)) === null || _a === void 0 ? void 0 : _a.origin);
            }
        }
    }, undefined, formControlInject);
    const select = useMemo(() => (dMultiple ? new Set(_select) : _select), [_select, dMultiple]);
    renderNodes.forEach((node) => {
        node.updateStatus(select);
    });
    const size = dSize !== null && dSize !== void 0 ? dSize : gSize;
    const disabled = (_a = (dDisabled || gDisabled || (dFormControl === null || dFormControl === void 0 ? void 0 : dFormControl.control.disabled))) !== null && _a !== void 0 ? _a : false;
    const hasSearch = searchValue.length > 0;
    const hasSelected = dMultiple ? select.size > 0 : !isNull(select);
    const [focusVisible, setFocusVisible] = useState(false);
    const filterFn = useCallback((item, searchStr = searchValue) => {
        const defaultFilterFn = (item) => {
            return item.label.includes(searchStr);
        };
        return dCustomSearch && dCustomSearch.filter ? dCustomSearch.filter(searchStr, item) : defaultFilterFn(item);
    }, [dCustomSearch, searchValue]);
    const sortFn = dCustomSearch === null || dCustomSearch === void 0 ? void 0 : dCustomSearch.sort;
    const searchList = useMemo(() => {
        if (!hasSearch) {
            return [];
        }
        const searchList = [];
        const reduceNodes = (nodes) => {
            nodes.forEach((node) => {
                if ((!dMultiple && !dOnlyLeafSelectable) || node.isLeaf) {
                    if (filterFn(node.origin)) {
                        searchList.push({
                            label: getText(node),
                            value: node.id,
                            disabled: node.disabled,
                            [TREE_NODE_KEY]: node,
                        });
                    }
                }
                if (node.children) {
                    reduceNodes(node.children);
                }
            });
        };
        reduceNodes(renderNodes);
        if (sortFn) {
            searchList.sort((a, b) => sortFn(a[TREE_NODE_KEY].origin, b[TREE_NODE_KEY].origin));
        }
        return searchList;
    }, [dMultiple, dOnlyLeafSelectable, filterFn, hasSearch, renderNodes, sortFn]);
    const [_noSearchFocusItem, setNoSearchFocusItem] = useState();
    const noSearchFocusItem = (() => {
        if (_noSearchFocusItem) {
            const node = nodesMap.get(_noSearchFocusItem.id);
            if (node && node.enabled) {
                return node;
            }
        }
        if (hasSelected) {
            return findNested(renderNodes, (node) => node.enabled && node.checked);
        }
    })();
    const [_searchFocusItem, setSearchFocusItem] = useState();
    const searchFocusItem = (() => {
        if (_searchFocusItem && findNested(searchList, (item) => item[TREE_NODE_KEY].enabled && item.value === _searchFocusItem.value)) {
            return _searchFocusItem;
        }
        if (hasSearch) {
            return findNested(searchList, (item) => item[TREE_NODE_KEY].enabled);
        }
    })();
    const handleClear = () => {
        onClear === null || onClear === void 0 ? void 0 : onClear();
        if (dMultiple) {
            changeSelect([]);
        }
        else {
            changeSelect(null);
        }
    };
    const [popupPositionStyle, setPopupPositionStyle] = useState({
        top: '-200vh',
        left: '-200vw',
    });
    const [transformOrigin, setTransformOrigin] = useState();
    const updatePosition = useEventCallback(() => {
        if (visible && boxRef.current && popupRef.current) {
            const height = popupRef.current.offsetHeight;
            const maxWidth = window.innerWidth - WINDOW_SPACE * 2;
            const width = Math.min(popupRef.current.scrollWidth, maxWidth);
            const { top, left, transformOrigin } = getVerticalSidePosition(boxRef.current, { width, height }, {
                placement: 'bottom-left',
                inWindow: WINDOW_SPACE,
            });
            setPopupPositionStyle({
                top,
                left,
                maxWidth,
            });
            setTransformOrigin(transformOrigin);
        }
    });
    useImperativeHandle(ref, () => ({
        updatePosition,
    }), [updatePosition]);
    const [selectedNode, suffixNode, selectedLabel] = (() => {
        let selectedNode = null;
        let suffixNode = null;
        let selectedLabel;
        if (dMultiple) {
            const selectedNodes = [];
            for (const v of _select) {
                const node = nodesMap.get(v);
                if (node) {
                    selectedNodes.push(node);
                }
                else {
                    console.warn(`Can't find item that value field is ${v}!`);
                }
            }
            suffixNode = (_jsx(DDropdown, Object.assign({ dList: selectedNodes.map((node) => {
                    const { value: itemValue, disabled: itemDisabled } = node.origin;
                    const text = dCustomSelected ? dCustomSelected(node.origin) : getText(node);
                    return {
                        id: itemValue,
                        label: text,
                        type: 'item',
                        disabled: itemDisabled,
                        node,
                    };
                }), onItemClick: (id, item) => {
                    const checkeds = item.node.changeStatus('UNCHECKED', select);
                    changeSelect(Array.from(checkeds.keys()));
                    return false;
                } }, { children: _jsx(DTag, Object.assign({ className: `${dPrefix}cascader__multiple-count`, tabIndex: -1, dSize: size }, { children: select.size })) })));
            selectedNode = selectedNodes.map((node) => (_jsxs(DTag, Object.assign({ className: `${dPrefix}cascader__multiple-tag`, dSize: size }, { children: [dCustomSelected ? dCustomSelected(node.origin) : node.origin.label, !(node.disabled || disabled) && (_jsx("div", Object.assign({ className: `${dPrefix}cascader__close`, role: "button", "aria-label": t('Close'), onClick: (e) => {
                            e.stopPropagation();
                            const checkeds = node.changeStatus('UNCHECKED', select);
                            changeSelect(Array.from(checkeds.keys()));
                        } }, { children: _jsx(CloseOutlined, {}) })))] }), node.id)));
        }
        else {
            if (!isNull(select)) {
                const node = nodesMap.get(select);
                if (node) {
                    selectedLabel = getText(node);
                    selectedNode = dCustomSelected ? dCustomSelected(node.origin) : selectedLabel;
                }
                else {
                    console.warn(`Can't find item that value field is ${select}!`);
                }
            }
        }
        return [selectedNode, suffixNode, selectedLabel];
    })();
    return (_jsx(DSelectbox, Object.assign({}, restProps, { onClick: (e) => {
            var _a;
            (_a = restProps.onClick) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
            changeVisible((draft) => (dSearchable ? true : !draft));
        }, dRef: {
            box: boxRef,
            input: dRef === null || dRef === void 0 ? void 0 : dRef.input,
        }, dClassNamePrefix: "cascader", dFormControl: dFormControl, dVisible: visible, dContent: hasSelected && selectedNode, dContentTitle: selectedLabel, dPlaceholder: dPlaceholder, dSuffix: suffixNode, dSize: size, dLoading: dLoading, dSearchable: dSearchable, dClearable: dClearable, dDisabled: disabled, dUpdatePosition: {
            fn: updatePosition,
            popupRef,
            extraScrollRefs: [],
        }, dInputRender: (el) => (_jsx(DComboboxKeyboard, Object.assign({ dVisible: visible, dEditable: dSearchable, dHasSub: !hasSearch, onVisibleChange: changeVisible, onFocusChange: (key) => {
                var _a;
                (_a = comboboxKeyDownRef.current) === null || _a === void 0 ? void 0 : _a.call(comboboxKeyDownRef, key);
            } }, { children: ({ render: renderComboboxKeyboard }) => {
                const input = renderComboboxKeyboard(cloneHTMLElement(el, {
                    value: searchValue,
                    'aria-controls': listId,
                    onBlur: (e) => {
                        var _a, _b;
                        (_b = (_a = el.props).onBlur) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                        changeVisible(false);
                    },
                    onKeyDown: (e) => {
                        var _a, _b, _c;
                        (_b = (_a = el.props).onKeyDown) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                        if ((e.code === 'Enter' || (!dSearchable && e.code === 'Space')) && visible) {
                            (_c = comboboxKeyDownRef.current) === null || _c === void 0 ? void 0 : _c.call(comboboxKeyDownRef, 'click');
                        }
                    },
                    onChange: (e) => {
                        var _a, _b;
                        (_b = (_a = el.props).onChange) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                        const val = e.currentTarget.value;
                        if (dSearchable) {
                            changeSearchValue(val);
                        }
                    },
                }));
                return isUndefined(dInputRender) ? input : dInputRender(input);
            } }))), onFocusVisibleChange: setFocusVisible, onClear: handleClear }, { children: ({ renderPopup }) => (_jsx(DTransition, Object.assign({ dIn: visible, dDuring: TTANSITION_DURING_POPUP, onEnter: updatePosition, afterEnter: () => {
                afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(true);
            }, afterLeave: () => {
                afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(false);
            } }, { children: (state) => {
                let transitionStyle = {};
                switch (state) {
                    case 'enter':
                        transitionStyle = { transform: 'scaleY(0.7)', opacity: 0 };
                        break;
                    case 'entering':
                        transitionStyle = {
                            transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING_POPUP}ms ease-out`).join(', '),
                            transformOrigin,
                        };
                        break;
                    case 'leaving':
                        transitionStyle = {
                            transform: 'scaleY(0.7)',
                            opacity: 0,
                            transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING_POPUP}ms ease-in`).join(', '),
                            transformOrigin,
                        };
                        break;
                    case 'leaved':
                        transitionStyle = { display: 'none' };
                        break;
                    default:
                        break;
                }
                return renderPopup(_jsxs("div", Object.assign({ ref: popupRef, className: getClassName(dPopupClassName, `${dPrefix}cascader__popup`), style: Object.assign(Object.assign({}, popupPositionStyle), transitionStyle) }, { children: [dLoading && (_jsx("div", Object.assign({ className: getClassName(`${dPrefix}cascader__loading`, {
                                [`${dPrefix}cascader__loading--empty`]: (hasSearch ? searchList : renderNodes).length === 0,
                            }) }, { children: _jsx(LoadingOutlined, { dSize: (hasSearch ? searchList : renderNodes).length === 0 ? 18 : 24, dSpin: true }) }))), hasSearch ? (_jsx(DTreeSearchPanel, { ref: comboboxKeyDownRef, id: listId, style: { pointerEvents: dLoading ? 'none' : undefined }, dGetItemId: getItemId, dList: searchList, dFocusItem: searchFocusItem, dCustomItem: dCustomItem, dMultiple: dMultiple, dOnlyLeafSelectable: dOnlyLeafSelectable, dFocusVisible: focusVisible, dVirtual: dVirtual, onFocusChange: (item) => {
                                if (!dataRef.current.focusList.has(item.value)) {
                                    dataRef.current.focusList.add(item.value);
                                    onFirstFocus === null || onFirstFocus === void 0 ? void 0 : onFirstFocus(item.value, item[TREE_NODE_KEY].origin);
                                }
                                setSearchFocusItem(item);
                            }, onClickItem: (item) => {
                                if (dMultiple) {
                                    const checkeds = item[TREE_NODE_KEY].changeStatus(item[TREE_NODE_KEY].checked ? 'UNCHECKED' : 'CHECKED', select);
                                    changeSelect(Array.from(checkeds.keys()));
                                }
                                else {
                                    changeSelect(item[TREE_NODE_KEY].id);
                                    changeVisible(false);
                                }
                            } })) : (_jsx(DList, { ref: comboboxKeyDownRef, id: listId, style: { pointerEvents: dLoading ? 'none' : undefined }, dGetItemId: getItemId, dList: renderNodes, dFocusItem: noSearchFocusItem, dCustomItem: dCustomItem, dMultiple: dMultiple, dVirtual: dVirtual, dFocusVisible: focusVisible, dRoot: true, onFocusChange: (node) => {
                                if (!dataRef.current.focusList.has(node.id)) {
                                    dataRef.current.focusList.add(node.id);
                                    onFirstFocus === null || onFirstFocus === void 0 ? void 0 : onFirstFocus(node.id, node.origin);
                                }
                                setNoSearchFocusItem(node);
                            }, onClickItem: (node) => {
                                if (dMultiple) {
                                    const checkeds = node.changeStatus(node.checked ? 'UNCHECKED' : 'CHECKED', select);
                                    changeSelect(Array.from(checkeds.keys()));
                                }
                                else {
                                    if (!dOnlyLeafSelectable || node.isLeaf) {
                                        changeSelect(node.id);
                                    }
                                    if (node.isLeaf) {
                                        changeVisible(false);
                                    }
                                }
                            } }))] })));
            } }))) })));
}
export const DCascader = React.forwardRef(Cascader);
