import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useImperativeHandle, useMemo, useRef, useState } from 'react';
import { first, fromEvent, Subject, takeUntil } from 'rxjs';
import { useAsync, useUnmount } from '@react-devui/hooks';
import { DoubleLeftOutlined, DoubleRightOutlined, LeftOutlined, RightOutlined } from '@react-devui/icons';
import { getClassName } from '@react-devui/utils';
import dayjs from '../../dayjs';
import { usePrefixConfig, useTranslation } from '../root';
function Panel(props, ref) {
    const { dDateCurrentSelected, dDateAnotherSelected, dConfigDate, onDateChange } = props;
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    const dataRef = useRef({});
    const async = useAsync();
    const [t, lang] = useTranslation();
    const onDestroy$ = useMemo(() => new Subject(), []);
    const dayjsCurrentSelected = dDateCurrentSelected
        ? dayjs(dDateCurrentSelected)
        : dayjs().set('hour', 0).set('minute', 0).set('second', 0);
    const dayjsAnotherSelected = dDateAnotherSelected ? dayjs(dDateAnotherSelected) : null;
    const [hoverDate, setHoverDate] = useState(null);
    const [_showDate, setShowDate] = useState(dayjsCurrentSelected.toDate());
    const showDate = dayjs(_showDate);
    const handleButtonMouseUp = () => {
        var _a, _b, _c, _e;
        (_b = (_a = dataRef.current).clearLoop) === null || _b === void 0 ? void 0 : _b.call(_a);
        (_e = (_c = dataRef.current).clearTid) === null || _e === void 0 ? void 0 : _e.call(_c);
    };
    const handleButtonDown = (unit, value) => {
        const loop = () => {
            setShowDate((draft) => {
                const d = dayjs(draft);
                return d.set(unit, d.get(unit) + value).toDate();
            });
            dataRef.current.clearLoop = async.setTimeout(() => loop(), 50);
        };
        dataRef.current.clearTid = async.setTimeout(() => loop(), 400);
        fromEvent(window, 'mouseup')
            .pipe(takeUntil(onDestroy$), first())
            .subscribe({
            next: () => {
                handleButtonMouseUp();
            },
        });
    };
    const getButtonProps = (unit, value) => ({
        onMouseDown: (e) => {
            if (e.button === 0) {
                handleButtonDown(unit, value);
            }
        },
        onTouchStart: () => {
            handleButtonDown(unit, value);
        },
        onTouchEnd: () => {
            handleButtonMouseUp();
        },
        onClick: () => {
            setShowDate((draft) => {
                const d = dayjs(draft);
                return d.set(unit, d.get(unit) + value).toDate();
            });
        },
    });
    const weekList = (() => {
        const firstDay = showDate.set('date', 1);
        const month = [];
        let week = [];
        for (let num = 0, addDay = -firstDay.day(); num < 7 * 6; num++, addDay++) {
            week.push(firstDay.add(addDay, 'day').toDate());
            if (week.length === 7) {
                month.push(week);
                week = [];
            }
        }
        return month;
    })();
    useUnmount(() => {
        handleButtonMouseUp();
        onDestroy$.next();
        onDestroy$.complete();
    });
    useImperativeHandle(ref, () => setShowDate, []);
    return (_jsxs("div", Object.assign({ className: `${dPrefix}date-picker__panel` }, { children: [_jsxs("div", Object.assign({ className: `${dPrefix}date-picker__header` }, { children: [_jsx("button", Object.assign({}, getButtonProps('year', -1), { className: `${dPrefix}date-picker__header-button`, title: t('DatePicker', 'Previous year') }, { children: _jsx(DoubleLeftOutlined, {}) })), _jsx("button", Object.assign({}, getButtonProps('month', -1), { className: `${dPrefix}date-picker__header-button`, title: t('DatePicker', 'Previous month') }, { children: _jsx(LeftOutlined, {}) })), _jsx("span", Object.assign({ className: `${dPrefix}date-picker__header-content` }, { children: showDate.format(lang === 'zh-CN' ? 'YYYY年 M月' : 'MMM YYYY') })), _jsx("button", Object.assign({}, getButtonProps('month', 1), { className: `${dPrefix}date-picker__header-button`, title: t('DatePicker', 'Next month') }, { children: _jsx(RightOutlined, {}) })), _jsx("button", Object.assign({}, getButtonProps('year', 1), { className: `${dPrefix}date-picker__header-button`, title: t('DatePicker', 'Next year') }, { children: _jsx(DoubleRightOutlined, {}) }))] })), _jsxs("table", Object.assign({ className: `${dPrefix}date-picker__content` }, { children: [_jsx("thead", { children: _jsx("tr", { children: dayjs.weekdaysMin().map((day) => (_jsx("th", { children: day }, day))) }) }), _jsx("tbody", { children: weekList.map((week, index1) => (_jsx("tr", { children: week.map((_d, index2) => {
                                var _a;
                                const d = dayjs(_d);
                                const { disabled } = (_a = dConfigDate === null || dConfigDate === void 0 ? void 0 : dConfigDate(_d)) !== null && _a !== void 0 ? _a : {};
                                const isCurrentSelected = dDateCurrentSelected && d.isSame(dayjsCurrentSelected, 'date');
                                const isAnotherSelected = dayjsAnotherSelected && d.isSame(dayjsAnotherSelected, 'date');
                                const isHover = !isCurrentSelected && !isAnotherSelected && dayjsAnotherSelected && d.isSame(hoverDate, 'date');
                                const isBetween = !isCurrentSelected &&
                                    !isCurrentSelected &&
                                    !isHover &&
                                    dDateCurrentSelected &&
                                    dayjsAnotherSelected &&
                                    d.isBetween(dayjsCurrentSelected, dayjsAnotherSelected, 'date');
                                const isBetweenHover = !isCurrentSelected && hoverDate && dayjsAnotherSelected && d.isBetween(hoverDate, dayjsAnotherSelected, 'date');
                                return (_jsx("td", Object.assign({ style: { pointerEvents: disabled ? 'none' : undefined }, onClick: () => {
                                        setShowDate(_d);
                                        onDateChange(_d);
                                    }, onMouseEnter: () => {
                                        var _a, _b;
                                        (_b = (_a = dataRef.current).clearHoverTid) === null || _b === void 0 ? void 0 : _b.call(_a);
                                        setHoverDate(_d);
                                    }, onMouseLeave: () => {
                                        var _a, _b;
                                        (_b = (_a = dataRef.current).clearHoverTid) === null || _b === void 0 ? void 0 : _b.call(_a);
                                        dataRef.current.clearHoverTid = async.setTimeout(() => setHoverDate(null), 50);
                                    } }, { children: _jsx("div", Object.assign({ className: getClassName(`${dPrefix}date-picker__cell`, {
                                            [`${dPrefix}date-picker__cell--out-month`]: d.get('month') !== showDate.get('month'),
                                            [`${dPrefix}date-picker__cell--today`]: !isCurrentSelected && !isAnotherSelected && d.isSame(dayjs(), 'date'),
                                            'is-current-selected': isCurrentSelected,
                                            'is-another-selected': !isCurrentSelected && isAnotherSelected,
                                            'is-hover': isHover,
                                            'is-between': isBetween,
                                            'is-between-hover': isBetweenHover,
                                            'is-disabled': disabled,
                                        }) }, { children: d.get('date') })) }), index2));
                            }) }, index1))) })] }))] })));
}
export const DPanel = React.forwardRef(Panel);
